use std::collections::HashMap;
use std::fmt::format;
use std::fs::write;
use std::io::Read;
use std::ops::Add;
use reqwest::header;
use serde::Deserialize;
use rss::{Channel, Item, Source};
use rss::ChannelBuilder;
use rss::ItemBuilder;
use build_html::{self, Html, HtmlContainer};

#[derive(Debug, Deserialize)]
struct SearchItem {
    gcode : String,
    gname : String,
    thumb_url : String,
    maker_name : String,

    max_price : i32,
    releasedate : String
}

#[derive(Debug, Deserialize)]
struct SearchResponse {
    items: Vec<SearchItem>
}

fn main() {
    let mut headers = header::HeaderMap::new();
    headers.insert("X-User-Key", header::HeaderValue::from_static("amiami_dev"));

    let client = reqwest::blocking::Client::builder()
        .default_headers(headers)
        .build().unwrap();

    let resp = client.get("https://api.amiami.com/api/v1.0/items?pagemax=50&lang=eng&s_st_list_preorder_available=1&s_st_list_newitem_available=1&s_cate_tag=14")
        .send()
        .unwrap()
        .json::<SearchResponse>()
        .unwrap();

    let mut rss_items = Vec::<Item>::new();

    for item in resp.items {
        let mut rss_item = Item::default();
        rss_item.set_title(item.gname.clone());
        rss_item.set_author(item.maker_name.clone());
        rss_item.set_link(format!("https://www.amiami.com/eng/detail/?gcode={}", item.gcode));

        let page = build_html::HtmlPage::new()
            .with_image(format!("https://img.amiami.com{}", item.thumb_url), item.gname)
            .with_paragraph(format!("{} JPY", item.max_price))
            .with_paragraph(format!("Release Date: {}", item.releasedate))
            .with_link(format!("https://www.amiami.com/eng/detail/?gcode={}", item.gcode), "Store Page")
            .to_html_string();

        rss_item.set_content(page.to_string());

        rss_items.push(rss_item);
    }

    let channel = ChannelBuilder::default()
        .title("AmiAmi - New Items")
        .items(rss_items)
        .build();

    write("items.xml", channel.to_string());
}
